import { Injectable } from '@angular/core';
import { catchError } from 'rxjs/operators';
import { Observable } from 'rxjs/internal/Observable';
import { HttpClient, HttpHeaders, HttpParams } from '@angular/common/http';
import { SaleReport } from './sale-report';
import { ErrorLoggerService } from '../core/error-logger.service';

const httpOptions = {
  headers: new HttpHeaders({'Content-Type': 'application/json'})
};

const url = '/api/sale-report';
const serviceName = 'SaleReportService';

@Injectable({
  providedIn: 'root'
})
export class SaleReportService {

  constructor(private http: HttpClient, private log: ErrorLoggerService) {
  }

  get(startDate: string, finishDate): Observable<SaleReport> {
    const options = {params: new HttpParams()};
    if (startDate !== null) {
      options.params = options.params.set('s', startDate);
    }
    if (finishDate !== null) {
      options.params = options.params.set('f', finishDate);
    }
    return <Observable<SaleReport>>this.http.get<SaleReport>(url, options)
      .pipe(
        catchError(this.log.handleError(serviceName, 'get'))
      );
  }
}
